#include "globals.h"

/* global variables */
struct alt_up_dev up_dev;	// holds a pointer to each open I/O device
/* The globals below are written by interrupt service routines, so we have to declare 
 * them as volatile to avoid the compiler caching their values in registers */
volatile int buf_index_record, buf_index_play;		// audio variables

/* function prototypes */
void pushbutton_ISR(void *, unsigned int);
void audio_ISR(void *, unsigned int);

/********************************************************************************
 * This program demonstrates use of HAL functions and interrupts
 *
 * It performs the following: 
 *  1. records audio for about 10 seconds when an interrupt is generated by
 *  	   pressing KEY[1]. LEDR[0] is lit while recording. Audio recording is 
 *  	   controlled by using interrupts
 * 	2. plays the recorded audio when an interrupt is generated by pressing
 * 	   KEY[2]. LEDR[1] is lit while playing. Audio playback is controlled by 
 * 	   using interrupts
********************************************************************************/
int main(void)
{
    /* The base addresses of devices are listed in the "BSP/system.h" file*/
        
	/* write to the pushbutton interrupt mask register, and set 3 mask bits to 1 
	 * (we can't use pushbutton 0; it is the Nios II reset button) */
    IOWR_ALTERA_AVALON_PIO_IRQ_MASK(PUSHBUTTONS_BASE, 0xE);
    
    /* Reset the edge capture register. */
    IOWR_ALTERA_AVALON_PIO_EDGE_CAP(PUSHBUTTONS_BASE, 0xE);  
    
	/* declare device driver pointers for devices */
	alt_up_audio_dev *audio_dev;

	// open the audio port
	audio_dev = alt_up_audio_open_dev ("/dev/Audio_Subsystem_Audio");
	if (audio_dev == NULL)
	{
		alt_printf ("Error: could not open audio device\n");
		return -1;
	}
	else
	{
		alt_printf ("Opened audio device\n");
		up_dev.audio_dev = audio_dev;	// store for use by ISRs
	}
	/* use the HAL facility for registering interrupt service routines. */
	/* Note: we are passsing a pointer to up_dev to each ISR (using the HAL context argument) as 
	 * a way of giving the ISR a pointer to every open device. This is useful because some of the
	 * ISRs need to access more than just one device (e.g. the pushbutton ISR accesses both
	 * the pushbutton device and the audio device) */
	alt_irq_register (1, (void *) &up_dev, (void *) pushbutton_ISR);   
	alt_irq_register (6, (void *) &up_dev, (void *) audio_ISR);

	while(1);								// main program simply idles, further program actions are handled by interrupts
    return 0;
}
